#!perl -w

#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2008 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################


#################################################################################
# This file expects the name of a parameter file its first and only argument
##
# It will return
# 0 The test executed correctly
# 1 The user interface could not be run
# 5 The project could not be built
#################################################################################

use warnings;
use strict;
## for debug
$main::SCRIPTDIR=$ENV{TESTRTDIR}."/lib/scripts/";
require $main::SCRIPTDIR . "Fcts.pl";
# Set to one to enable debugging
my $DEBUG = 0;
if(defined($ENV{TESTRTRQMLOG})) {
	$DEBUG =1;
	open (DEBUGLOG,">>$ENV{TESTRTRQMLOG}") || die &PerlTranslate("TestRTRqm.pl", "Error: Unable write on log file %1\n", "%1 is the log file",$ENV{TESTRTRQMLOG});
}	
# The script expects one parameter which is the name of the parameter file
die "$0 needs one, and only one parameter which is the name of the file containing the parameters to run the test\n" unless (@ARGV == 1);
my $paramFile = $ARGV[0];
my $Drive;
my $resultsFile;
my $htmlReport;
my $BuildlogFile;
my $studioMode=0;
my $errMount="";

$DEBUG && print DEBUGLOG GetDate(), " Read of $paramFile:\n";

# Does the parameter file exist and is it readable?
open (IN, $paramFile) || die "Cannot open the file $paramFile for reading:\n$!\n";

my %params;
my $lineno = 0;
while (<IN>) {
  $DEBUG && print DEBUGLOG ;
  $lineno++;
  next if (/^\s*$/); # Ignore empty lines
  next if (/^\s*#/); # Ignore comments
  chomp; # Remove newline characters
  # now must have a line with the format key=value
  if (m,\s*(\w+)=(.+)\s*$,) {
    my $key = $1;
    my $value = $2;
    # Store the key and value in the parameter lookup table
    $params{$key} = $value;
  } else {
    $DEBUG && print DEBUGLOG "Unrecognized line in the parameter file at line $lineno\n";
    die "Unrecognized line in the parameter file\n$lineno: $_\n";
  }
}
close (IN);
# key used by the RQM adapter:
#testRTScriptPath is the project file .rtp (args[0])
#testNodeName is the node name to rebuild (args[1]) (filter)
#tdpName is the configuration Name (args[2])
#sharedPrefix is the part of the path that can be shared between server and TestRT install (args[3])
#relativePath is not used here (args[4])
#tempDirName is the temporary directory used to generate html report
#resultSummaryFile is the xml file giving the summary of the executed tests 

# Check that we have received the minimum expected keys
#die "Missing the name of the test node to execute. Expected parameter name is \"testNodeName\".\n" unless (exists $params{testNodeName});
die "Missing the name of the project file containing the tests. Expected parameter name is \"testRTScriptPath\".\n" unless (exists $params{testRTScriptPath});

# Remove double backslashes from the name of the project
if($params{sharedPrefix}) {
	$params{sharedPrefix} =~ s/\\\\/\\/g;
	$params{sharedPrefix} =~ s/\\:/:/g;
	if(($params{sharedPrefix} =~ m/^\\\\/) && ($^O =~ /Win32/)) {
		my $mount="net use * ".$params{sharedPrefix};
		$DEBUG && print DEBUGLOG GetDate(), " Running the following command:\n$mount\n";
		my $out = `$mount` ;
		my $status = $? ;
		my $ret = $status / 256;
		if(($out=~ m/\s(\w\:)\s/) && ($ret ==0)){
			$Drive=$1;
		} else {
			$errMount = $out." error code ".$ret; 		
		}
	}
}

$params{testRTScriptPath} =~ s/\\\\/\\/g;
$params{testRTScriptPath} =~ s/\\:/:/g;
$params{testRTScriptPath} =~ s/^\Q$params{sharedPrefix}\E/$Drive/ if($Drive);

if($params{tempDirName}) {
	$params{tempDirName} =~ s/\\\\/\\/g;
	$params{tempDirName} =~ s/\\:/:/g;
	$params{tempDirName} =~ s/^\Q$params{sharedPrefix}\E/$Drive/ if($Drive);
}	

# Calculate the log file name
my $logfile =$params{tempDirName}.'/TestRTRqm.log';
$DEBUG && print DEBUGLOG "\n", GetDate(), " logfile = $logfile\n";
open (LOG, ">$logfile") || die "Cannot open the log file $logfile\n";
print LOG "# set environment variable TESTRTRQMLOG to a logFileName.txt to get more details\n";
print LOG $errMount;
# Now do what you want!

# Then run the test via studio or Visual Test
my $command;

if($params{testRTScriptPath}=~m/\.rtp$/){
	$studioMode=1;
	if($params{testNodeName}) {
		$command = "studio -r \"".$params{testNodeName}."\" \"".$params{testRTScriptPath}."\"";
	} else {
		$command = "studio -r  \"".$params{testRTScriptPath}."\"";
	}
	$command .= " -config \"".$params{tdpName}."\"" if (exists $params{tdpName});
	$command .=  " -html \"".$params{tempDirName}."/htmlDir\"" if (-d $params{tempDirName});
	$htmlReport=$params{tempDirName}.'/htmlDir' if (-d $params{tempDirName});
	#get global summary into 1 result file for futur Adapter release
	#for now the name of this global summary must not be named xxx.summary.txt otherwise it will be read as node result in addition to all other summary
	#globalize the test result in 1 file
	$resultsFile = $params{testRTScriptPath};
	$resultsFile =~s/([\\\/])[^\\\/]*$/$1RQM.results.txt/i;
	$ENV{RTRT_TEST_SUM}=$resultsFile;
	unlink($resultsFile);
} elsif ($params{testRTScriptPath}=~m/\.test_suite$/){
	my $TESTSUITE=$params{testRTScriptPath};
	my $WORKSPACE;
	my @dirs=split('[\\\/]',$TESTSUITE);
	$WORKSPACE=shift(@dirs);
	while(!(-d $WORKSPACE.'/.metadata') && @dirs){
		$WORKSPACE.='/'.shift(@dirs);
	}
	die "WorkSpace Not found in $TESTSUITE\n" unless(@dirs);
	$WORKSPACE='"'.$WORKSPACE.'"' if($WORKSPACE=~/\s/);
	$TESTSUITE ='/'.join('/',@dirs);# make it relative to workspace
	$TESTSUITE='"'.$TESTSUITE.'"' if($TESTSUITE=~/\s/);
	$command .='"'.$ENV{TESTRTDIR}.'/eclipse" -nosplash -application com.ibm.rational.testrt.test.campaign.launcher.TestLaunchers ';
	$command .='-data '.$WORKSPACE.' ';
	$command .=$TESTSUITE.' ';
	$command .="-xmlfile=\"$params{tempDirName}/$params{resultSummaryFile}\" ";
	$command .="-XMLREPORTS ";
	#-BINDIR="<Rpertoire d'install des binaires TestRT>"
	#-TDPDIR="<Liste de rpertoire o se trouve les TDPs>"
} else {
	die "Unknown project file or test_suite. The extension does not match the Expected one in \"testRTScriptPath\".\n" ;
}
$DEBUG && print DEBUGLOG GetDate(), " Running the following command:\n$command\n";
print LOG GetDate(), " Running the following command:\n$command\n";
$! = 0;
my $status = system ($command);
my $ret = $status / 256;
$DEBUG && print DEBUGLOG GetDate(), " Raw return status $status => $ret\n";
# Check the return value and return one of the following values from the script
# 0 The test executed correctly
# 1 studio could not be run
# 5 The project could not be built by studio
if($studioMode){
	if ($ret == 1) {
	  # There is an ambiguity with the value 1. 1 means an error occurred in system
	  # or the TestRT test was built successfully. $! will tell us which it is
	  if ($! eq "") {
		# TestRT test was built successfully
		$ret = 0;
	  } else {
		# an error occured in system call
		print LOG "TestRTRqm.pl: When trying to run the Test RealTime user interface the following error code was returned: $ret\n$!\n\n";
		$ret =1;
	  }
	} elsif ($ret == 5) {
	  # Build failed
	  print LOG "TestRTRqm.pl: Build of the test failed with error code 5\n";
	} else {
	  print LOG "TestRTRqm.pl: When trying to run the Test RealTime user interface the following error code was returned: $ret\n$\n!\n" unless ($ret == 0);
	  $ret = 1;
	}
}
else {
	if ($ret != 0) {
		print LOG "TestRTRqm.pl: Build of the test failed with error code $ret\n";
	}
}

if($studioMode){
  if(-f $resultsFile) {
    if(open(RES,$resultsFile)) {
        open(XML, ">$params{tempDirName}/$params{resultSummaryFile}") or die "TestRTRqm.pl: unable to write the result summary file $params{resultSummaryFile}\n";
        print XML "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<RUNS>\n";
        print XML "<LOG>$BuildlogFile</LOG>\n" if($BuildlogFile);
        my @COVFILES;
        my ($TestNodeName,$TestPass,$TestFail,$TestInco,$TestTotal,$TestPath,$TestReport,$branch,$cov);
        $TestInco=0;
	$branch=0;
	$cov=0;
	while(<RES>){
	    if(/^Test node=(.*)$/){$TestNodeName=$1;}
	    if(/^Report file=(.*)$/){$TestReport=$1;$TestPath=$1;$TestPath=~s/[\\\/][^\\\/]\.xrd$//;}
	    if(/^Passed=(.*)$/) {$TestPass=$1;}
	    if(/^Failed=(.*)$/) {$TestFail=$1;}
	    if(/^Inconclusive=(.*)$/) {$TestInco=$1;}
	    if(/^(FDCFILES|TIOFILES)=(.*)$/) {push(@COVFILES,$2);}
	    if(/^Total=(.*)$/) {
	        $TestTotal=$1;
	        my $status="OK";
	        $status="INCONCLUSIVE" if($TestInco);
	        $status="KO" if($TestFail);
#<RUN NAME="th_add" PATH="/prj/th_add.test_harness" DATE="Thu Sep 22 15:25:12 CEST 2011" PASSED="0" INCONCLUSIVE="0" FAILED="1" STATUS="KO" >
#	<REPORT>C:/temp/runtime-New_configuration33/prj/TestRT_win32/th_add.1316697912045.test_report</REPORT>
#</RUN>	        
	        print XML "<RUN NAME=\"$TestNodeName\" PATH=\"$TestPath\" DATE=\"",GetDate();
	        print XML "\" PASSED=\"$TestPass\" FAILED=\"$TestFail\" INCONCLUSIVE=\"$TestInco\" TOTAL=\"$TestTotal\" STATUS=\"$status\" >\n";
	        print XML "   <REPORT>$TestReport</REPORT>\n";
	        print XML "</RUN>\n";
	        $TestNodeName='';
	        $TestPath='';
	        $TestReport='';
	        $TestPass=0;
	        $TestFail=0;
	        $TestInco=0;
	        $TestTotal=0;
	    }
	}
	if(@COVFILES){
	   if(open(OPT,">attolcov.opt")){
	   	print OPT "@COVFILES\n";
	   	close OPT;
		$command="attolcov  -su=c -output=coverage_results.txt \@attolcov.opt";
		$DEBUG && print DEBUGLOG GetDate(), " Running the following command:\n$command\nattolcov.opt is:@COVFILES\n";
		$! = 0;
		my $status = system ($command);
		my $ret = $status / 256;
		$DEBUG && print DEBUGLOG GetDate(), " Raw return status $status => $ret\n";
		if(open(RCOV,"coverage_results.txt")){
		   while(<RCOV>){
		   	next if(/Multiple cond/);
			if(/\((\d+)\/(\d+)\)/){
			   $cov+=$1; $branch+=$2;
			}
		   }
		   close RCOV;
		   unlink("coverage_results.txt","attolcov.opt") unless($DEBUG);
		}
	   }
	}
	print XML "<COVERAGE_RESULTS COVERED_BRANCH=\"$cov\" BRANCH_NUMBER=\"$branch\" ></COVERAGE_RESULTS>\n" if($branch);
        print XML "</RUNS>\n";
        close XML;
	close RES;
	unlink($resultsFile) unless($DEBUG);
    } else {
        print LOG "TestRTRqm.pl: Unable to read the results file $resultsFile\n";
    }
  }
} else {
#Visual Test Mode
  if(open(XML, "$params{tempDirName}/$params{resultSummaryFile}") ) {
     while(<XML>){
         if(/<(REPORT|LOG)>(.+)<\/(REPORT|LOG)>/){
         	my ($d,$b,$e)=SplitFileName($2);
         	#copy($2,$params{tempDirName}.'/'.$b.'.'.$e);
			
			# Copy reports
			my @REPORTSEXT = ("test_report","coverage_report","mempro_report","perfpro_report");
			foreach (@REPORTSEXT) {
				my $report_src = $d.'/'.$b.'.'.$_;
				my $report_dest = $params{tempDirName}.'/'.$b.'.'.$_;
				
				if (-f $report_src) {
					copy($report_src, $report_dest);
				}
			}
			
			# Copy style sheets if not already done
			my @STYLESHEETS = ("testsuite_report","coverage_report","mempro_report","perfpro_report");
			foreach (@STYLESHEETS) {
				my $styleSheet_src = $d.'/'.$_.'.xsl';
				my $styleSheet_dest = $params{tempDirName}.'/'.$_.'.xsl';
			
				if (-f $styleSheet_src) {
					copy($styleSheet_src, $styleSheet_dest);
				}
			}
			
			
         }
     }
     close XML;
  } else {
     print LOG "TestRTRqm.pl: Unable to read the results file $params{tempDirName}/$params{resultSummaryFile}\n";
  }
}
close (LOG);
unlink($logfile) unless(-s $logfile);
system("net use $Drive /delete") if($Drive);
$DEBUG && print DEBUGLOG GetDate(), " exit code = $ret\n";
close DEBUGLOG;
exit($ret);

1;