##!perl -w
#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2011 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

# useful for stdout/stderr direct viewing inside 'console' of studio window
# (better: we have stdout/stderr lines in right order and not stderr before stdout
use FileHandle ;

STDOUT->autoflush(1);
STDERR->autoflush(1);

unless ($ENV{'TESTRTDIR'}) {
	print "Please set TESTRTDIR environment variable\n";
}
$SCRIPTDIR=$ENV{'TESTRTDIR'}.'/lib/scripts/';
# Miscellaneous functions 
require $SCRIPTDIR . "Fcts.pl";

my $appName="com.ibm.rational.testrt.test.campaign.launcher.TestLaunchers";
my @STYLESHEETS = ("testsuite_report","coverage_report","mempro_report","perfpro_report");

my $workspace="";
my $args="";
my @testsuitelist=();
my $testsuites="";
my $build="";
my $buildArgs="";
my $reportdir="";
while(@ARGV) {
    $_=shift(@ARGV);
	
	if(/^-WORKSPACE=(.+)/i){
		$workspace=$1;
	}
	elsif (/^-BUILD_PROJECT=(.+)/i) {
		$build="-build ".$1." "."-cleanBuild ".$1." ";
		$buildArgs="-BUILD_PROJECT=".$1." ";
	}
	elsif (/^-REPORTDIR=(.+)/i) {
		$reportdir=$1;
		$args.="-xmlfile=\"$reportdir/Summary.xml\" -XMLREPORTS ";
	}
	elsif (/^-BINDIR=(.+)/i || /^-TDPDIR=(.+)/i) {
		$args.=$_." ";
	}
	else {
		push(@testsuitelist,$_);
	}
}

if ($workspace eq "") {
	my @dirs=split('[\\\/]',$testsuitelist[0]);
	$workspace=shift(@dirs);
	while(!(-d $workspace.'/.metadata') && @dirs){
		$workspace.='/'.shift(@dirs);
	}
	unless(@dirs) {
		print "WorkSpace Not found in $testsuitelist[0]\n" ;
		print "\nUsage rtrteclipse:\n";
		print "\t[-WORKSPACE={workspace directory}]\n";
		print "\t[testsuite_pathname [{testsuite_pathname}]]\n";
		print "\t[-BUILD_PROJECT={project_name | all}]\n";
		print "\t[-BINDIR={directory}]\n";
		print "\t[-TDPDIR={directory}]\n";
		print "\t[-REPORTDIR={directory}]\n"; #directory where to copy all xml report file
		print "\ne.g: rtrteclipse -WORKSPACE={workspace} {testsuitePathFromWorkspace} [{testsuite}] [options]\n";
		print "\ne.g: rtrteclipse {testsuiteWithAbsolutePath} [{testsuite}] [options]\n";

		exit(1);
	}
	print "WorkSpace found as $workspace.\n";
	$testsuitelist[0]='/'.join('/',@dirs);# make it relative to workspace
}
$workspace='"'.$workspace.'"' if($workspace=~/\s/);
my $tspath=$testsuitelist[0]; $tspath=~s,[\\\/][^\\\/]*$,,;#get the path to the first ts
foreach my $ts (@testsuitelist){
	unless(-f $workspace.'/'.$ts || -f $workspace.'/'.$ts.'.test_suite') {
		if(-f $ts || -f $ts.'.test_suite'){
			$ts=~s,^\Q$workspace\E,,;
		} else {
			# try to reuse the path from the first test_suit
			$ts=$tspath.'/'.$ts if(-f $workspace.'/'.$tspath.'/'.$ts || $workspace.'/'.$tspath.'/'.$ts.'.test_suite')
		}
	}
	$ts='"'.$ts.'"' if($ts=~/\s/);
	$testsuites.=$ts.' ';
}
if($reportdir){
    if(-d $reportdir){
	unlink("$reportdir/Summary.xml");
    } else {
	mkdir($reportdir);
    }
}

if ($testsuites eq "" and $build eq "") { # No args means build all
	$build="-build all -cleanBuild all ";
	$buildArgs="-BUILD_PROJECT=all ";
}
	
my $retBuild=0;
if (!($build eq "")) { # Build application
	my $buildCommand = "\"$ENV{'TESTRTDIR'}/eclipsec\" -nosplash -application org.eclipse.cdt.managedbuilder.core.headlessbuild -data $workspace $build $args";
	print $buildCommand,"\n";
	$retBuild=system ($buildCommand);
#	print "status=$retBuild\n";
}

if ($retBuild==0) {
	my $command = "\"$ENV{'TESTRTDIR'}/eclipsec\" -nosplash -application $appName -data $workspace $testsuites $buildArgs $args";
	print $command,"\n";
	$retBuild=system ($command);
#	print "status=$retBuild\n";
	
	if(($reportdir ne '') && open(XML, "$reportdir/Summary.xml") ) {
	     open(OUT, ">$reportdir/Summary.xml.tmp");
	     while(<XML>){
	         my $line=$_;
	         if(/<(REPORT|LOG)>(.+)<\/(REPORT|LOG)>/){
	         	my $report_src = $2;
	         	my ($d,$b,$e)=SplitFileName($report_src);
			if($e eq 'log'){
				if (-f $report_src) {
					my $report_dest = $reportdir.'/'.$b.'.'.$e;
					copy($report_src,$report_dest);
					$line=~s/\Q$report_src\E/$report_dest/;	
				}
			} else {
				# Copy reports
				my @REPORTSEXT = ("test_report","coverage_report","mempro_report","perfpro_report");
				foreach (@REPORTSEXT) {
					my $report_other = $d.$b.'.'.$_;
					
					if (-f $report_other) {
						my $report_dest = $reportdir.'/'.$b.'.'.$_.'.xml';
						copy($report_other, $report_dest);
						$line=~s/\Q$report_src\E/$report_dest/ if($e eq $_);
#						print $line if($e eq $_);
					}
				}
				
				# Copy style sheets if not already done
				foreach (@STYLESHEETS) {
				    if($_ ne ''){
					my $styleSheet_src = $d.'/'.$_.'.xsl';
				
					if (-f $styleSheet_src) {
						my $styleSheet_dest = $reportdir.'/'.$_.'.xsl';
						unless(-f $styleSheet_dest) {
							copy($styleSheet_src, $styleSheet_dest);
						}
						$_='';
					}
				    }
				}
			}
	         } elsif(/<RUN\W/) {
	         	$retBuild =2 if(/\sSTATUS=.KO/);# exit code 3 if a test is KO
	         }
	         print OUT $line;
	     }
	     close XML; close OUT;
	     rename("$reportdir/Summary.xml.tmp","$reportdir/Summary.xml");
	} else {
		$retBuild =1;# exit code 1 if no execution
	}
}
#print join(',',@STYLESHEETS)," not copied\n";
exit($retBuild);

1;
